/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.List;
import java.util.Optional;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.server.LinkBuilder;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.system.SystemStatusService;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangController;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class RootViewLinkHandler {

	static final String VORGAENGE_ALL_REL_TEMPLATE = "vorgaenge_%s_all";
	static final String VORGAENGE_MY_REL_TEMPLATE = "vorgaenge_%s_my";
	static final String VORGAENGE_UNASSIGNED_REL_TEMPLATE = "vorgaenge_%s_unassigned";

	static final String ALL_VORGAENGE_REL = "vorgaenge_all";
	static final String MY_VORGEANGE_REL = "vorgaenge_my";
	static final String UNASSIGNED_VORGEANGE_REL = "vorgaenge_unassigned";

	static final String ALL_WIEDERVORLAGEN_REL = "vorgaenge_wiedervorlagen_all";
	static final String MY_WIEDERVORLAGEN_REL = "vorgaenge_wiedervorlagen_my";
	static final String UNASSIGNED_WIEDERVORLAGEN_REL = "vorgaenge_wiedervorlagen_unassigned";

	static final String SEARCH_ALL_REL = "search_all";
	static final String SEARCH_MY_VORGEANGE_REL = "search_my";
	static final String SEARCH_UNASSIGNED_VORGEANGE_REL = "search_unassigned";

	static final String ALL_UNGELESENE_NACHRICHTEN_REL = "vorgaenge_ungelesene_nachrichten_all";
	static final String MY_UNGELESENE_NACHRICHTEN_REL = "vorgaenge_ungelesene_nachrichten_my";
	static final String UNASSIGNED_UNGELESENE_NACHRICHTEN_REL = "vorgaenge_ungelesene_nachrichten_unassigned";

	static final int PAGE_SIZE = 100;

	private final CurrentUserService currentUserService;
	private final SystemStatusService systemStatusService;

	private final List<VorgangStatus> statusList = List.of(VorgangStatus.NEU, VorgangStatus.ANGENOMMEN, VorgangStatus.IN_BEARBEITUNG,
			VorgangStatus.BESCHIEDEN, VorgangStatus.ABGESCHLOSSEN, VorgangStatus.VERWORFEN, VorgangStatus.ZU_LOESCHEN);

	public void addViewLinks(EntityModel<Root> model, Optional<UserId> userId) {
		model.add(buildGetAllVorgaengeLink());
		addSearchAllVorgaengeLink(model);

		if (currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)) {
			addViewLinksForVerwaltungPoststelle(model, userId);
		}
		if (currentUserService.hasRole(UserRole.VERWALTUNG_USER)) {
			addViewLinksForVerwaltungUser(model, userId);
		}
	}

	private Link buildGetAllVorgaengeLink() {
		return linkTo(methodOn(VorgangController.class).getAll(0, PAGE_SIZE)).withRel(ALL_VORGAENGE_REL);
	}

	void addSearchAllVorgaengeLink(EntityModel<Root> model) {
		if (systemStatusService.isSearchServerAvailable()) {
			model.add(buildGetAllVorgaengeBySearchByLink());
		}
	}

	Link buildGetAllVorgaengeBySearchByLink() {
		return linkTo(methodOn(VorgangController.class).getAllBySearchBy(0, PAGE_SIZE, null)).withRel(SEARCH_ALL_REL);
	}

	void addViewLinksForVerwaltungPoststelle(EntityModel<Root> model, Optional<UserId> userId) {
		userId.map(this::buildMyVorgaengeLink).ifPresent(model::add);

		if (systemStatusService.isSearchServerAvailable()) {
			userId.map(this::buildSearchMyVorgaengeLink).ifPresent(model::add);
		}
	}

	void addViewLinksForVerwaltungUser(EntityModel<Root> model, Optional<UserId> userId) {
		addAllVorgangStatusLinks(model);

		model.add(buildGetAllUnassignedVorgaengeLink());
		model.add(buildUnassignedVorgaengeStatusLinks());

		userId.map(this::buildMyVorgaengeLink).ifPresent(model::add);
		userId.map(this::buildMyVorgaengeStatusLinks).ifPresent(model::add);

		if (systemStatusService.isSearchServerAvailable()) {
			userId.map(this::buildSearchMyVorgaengeLink).ifPresent(model::add);
			model.add(buildSearchUnassignedVorgaengeLink());
		}

		model.add(buildGetAllByHasNextWiedervorlageFristLink().withRel(ALL_WIEDERVORLAGEN_REL));
		model.add(buildGetAllByAssignedToAndHasNextWiedervorlageFristLink(UserId.empty(), UNASSIGNED_WIEDERVORLAGEN_REL));
		userId.map(id -> buildGetAllByAssignedToAndHasNextWiedervorlageFristLink(id, MY_WIEDERVORLAGEN_REL)).ifPresent(model::add);

		addGetByUngeleseneNachrichtenLinks(model, userId);
	}

	Link buildGetAllUnassignedVorgaengeLink() {
		return buildGetAllVorgaengeByAssignedToLink(UserId.empty()).withRel(UNASSIGNED_VORGEANGE_REL);
	}

	Link buildMyVorgaengeLink(@NonNull UserId userId) {
		return buildGetAllVorgaengeByAssignedToLink(userId).withRel(MY_VORGEANGE_REL);
	}

	LinkBuilder buildGetAllVorgaengeByAssignedToLink(UserId userId) {
		return linkTo(methodOn(VorgangController.class).getAllByAssignedTo(0, PAGE_SIZE, userId));
	}

	Link buildSearchMyVorgaengeLink(@NonNull UserId userId) {
		return buildGetAllVorgaengeByAssignedToAndSearchByLink(userId).withRel(SEARCH_MY_VORGEANGE_REL);
	}

	Link buildSearchUnassignedVorgaengeLink() {
		return buildGetAllVorgaengeByAssignedToAndSearchByLink(UserId.empty()).withRel(SEARCH_UNASSIGNED_VORGEANGE_REL);
	}

	LinkBuilder buildGetAllVorgaengeByAssignedToAndSearchByLink(UserId userId) {
		return linkTo(methodOn(VorgangController.class).getAllByAssignedToAndSearchBy(0, PAGE_SIZE, userId, null));
	}

	void addAllVorgangStatusLinks(EntityModel<Root> model) {
		addVorgangStatusLinks(model, VORGAENGE_ALL_REL_TEMPLATE);
	}

	void addVorgangStatusLinks(EntityModel<Root> model, String linkRelTemplate) {
		statusList.forEach(status -> model.add(buildGetAllVorgaengeByStatus(status, linkRelTemplate)));
	}

	Link buildGetAllVorgaengeByStatus(VorgangStatus status, String linkRelTemplate) {
		var linkRel = formatLinkRelTemplate(linkRelTemplate, status);
		return linkTo(methodOn(VorgangController.class).getAllByStatus(0, PAGE_SIZE, status)).withRel(linkRel);
	}

	List<Link> buildMyVorgaengeStatusLinks(@NonNull UserId userId) {
		return buildVorgangStatusLinks(VORGAENGE_MY_REL_TEMPLATE, userId);
	}

	List<Link> buildUnassignedVorgaengeStatusLinks() {
		return buildVorgangStatusLinks(VORGAENGE_UNASSIGNED_REL_TEMPLATE, UserId.empty());
	}

	List<Link> buildVorgangStatusLinks(String linkRelTemplate, UserId userId) {
		return statusList.stream().map(status -> buildGetAllVorgaengeByAssignedToAndStatus(status, linkRelTemplate, userId)).toList();
	}

	Link buildGetAllVorgaengeByAssignedToAndStatus(VorgangStatus status, String linkRelTemplate, UserId userId) {
		var linkRel = formatLinkRelTemplate(linkRelTemplate, status);
		return linkTo(methodOn(VorgangController.class).getAllByAssignedToAndStatus(0, PAGE_SIZE, userId, status)).withRel(linkRel);
	}

	String formatLinkRelTemplate(String linkRelTempalte, VorgangStatus status) {
		return String.format(linkRelTempalte, status.toString().toLowerCase());
	}

	Link buildGetAllByHasNextWiedervorlageFristLink() {
		return linkTo(methodOn(VorgangController.class).getAllByHasNextWiedervorlageFrist(0, PAGE_SIZE,
				VorgangController.PARAM_NEXT_WIEDERVORLAGE_FRIST_EXISTS)).withRel(ALL_WIEDERVORLAGEN_REL);
	}

	Link buildGetAllByAssignedToAndHasNextWiedervorlageFristLink(@NonNull UserId userId, String linkRel) {
		return linkTo(methodOn(VorgangController.class).getAllByAssignedToAndHasNextWiedervorlageFrist(0, PAGE_SIZE, userId,
				VorgangController.PARAM_NEXT_WIEDERVORLAGE_FRIST_EXISTS)).withRel(linkRel);
	}

	void addGetByUngeleseneNachrichtenLinks(EntityModel<Root> model, Optional<UserId> userId) {
		model.add(buildGelAllByUngeleseneNachrichtenLink());
		model.add(buildGetAllByAssignedToAndUngeleseneNachrichten(UserId.empty(), UNASSIGNED_UNGELESENE_NACHRICHTEN_REL));
		userId.map(id -> buildGetAllByAssignedToAndUngeleseneNachrichten(id, MY_UNGELESENE_NACHRICHTEN_REL)).ifPresent(model::add);
	}

	Link buildGelAllByUngeleseneNachrichtenLink() {
		return linkTo(methodOn(VorgangController.class)
				.getAllByUngeleseneNachrichten(0, PAGE_SIZE, VorgangController.PARAM_NACHRICHTEN_UNGELESENE))
						.withRel(ALL_UNGELESENE_NACHRICHTEN_REL);
	}

	Link buildGetAllByAssignedToAndUngeleseneNachrichten(@NonNull UserId userId, String linkRel) {
		return linkTo(methodOn(VorgangController.class)
				.getAllByAssignedToAndUngeleseneNachrichten(0, PAGE_SIZE, userId, VorgangController.PARAM_NACHRICHTEN_UNGELESENE))
						.withRel(linkRel);
	}
}