/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static de.ozgcloud.alfa.common.command.CommandController.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.errorhandling.ExceptionController;
import de.ozgcloud.alfa.common.errorhandling.ProblemDetailMapper;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.test.TestUtils;

class CommandControllerTest {

	@InjectMocks // NOSONAR
	private CommandController controller;
	@Mock
	private CommandService service;
	@Mock
	private CommandModelAssembler modelAssembler;
	@Mock
	private ProblemDetailMapper problemDetailMapper;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new ExceptionController(problemDetailMapper)).build();
	}

	@Nested
	class TestGetCommand {

		private final Command command = CommandTestFactory.create();

		@BeforeEach
		void initTest() {
			when(service.getById(anyString())).thenReturn(command);
		}

		@Test
		void shouldReturnOk() throws Exception {
			doRequest().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).getById(CommandTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			doRequest();

			verify(modelAssembler).toModel(command);
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(CommandController.COMMANDS_PATH + "/" + CommandTestFactory.ID)).andExpect(status().is2xxSuccessful());
		}
	}

	@Nested
	@DisplayName("Revoke Command")
	class TestRevokeCommand {

		@Test
		void shouldReturnOk() throws Exception {
			when(service.revoke(anyString())).thenReturn(CommandTestFactory.create());

			doRequest(CommandStatus.REVOKED).andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			when(service.revoke(anyString())).thenReturn(CommandTestFactory.create());

			doRequest(CommandStatus.REVOKED);

			verify(service).revoke(CommandTestFactory.ID);
		}

		@Test
		void shouldReturnForbiddenForOtherStatus() throws Exception {
			doRequest(CommandStatus.FINISHED).andExpect(status().isForbidden());
		}

		private ResultActions doRequest(CommandStatus statusToPatch) throws Exception {
			return mockMvc.perform(patch(COMMANDS_PATH + "/" + CommandTestFactory.ID).contentType(MediaType.APPLICATION_JSON)
					.content(TestUtils.loadTextFile("jsonTemplates/command/patchStatus.json", statusToPatch.name())));

		}
	}

	@Nested
	class TestExistsPendingCommands {

		@Test
		void shouldCallService() {
			controller.existsPendingCommands(VorgangHeaderTestFactory.ID);

			verify(service).existsPendingCommands(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestGetPendingCommands {

		private final Command command = CommandTestFactory.create();
		private final Stream<Command> pendingCommands = Stream.of(command);

		@BeforeEach
		void initTest() {
			when(service.getPendingCommands(anyString())).thenReturn(pendingCommands);
		}

		@Test
		void shouldReturnOk() throws Exception {
			doRequest().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).getPendingCommands(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			doRequest();

			verify(modelAssembler).toCollectionModel(pendingCommands);
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(CommandController.COMMANDS_PATH)
							.param(CommandController.PARAM_PENDING, Boolean.toString(true))
							.param(CommandController.PARAM_VORGANG_ID, VorgangHeaderTestFactory.ID))
					.andExpect(status().is2xxSuccessful());
		}
	}
}