/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.statistic;

import static de.ozgcloud.alfa.statistic.StatisticRemoteService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.AlfaProperties;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.vorgang.common.GrpcQueryOperator;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticQuery;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticQuery.GroupMethod;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticRequest;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticResponse;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticResult;
import de.ozgcloud.vorgang.statistic.StatisticServiceGrpc.StatisticServiceBlockingStub;

class StatisticRemoteServiceTest {

	@Spy
	@InjectMocks
	private StatisticRemoteService service;
	@Mock
	private StatisticServiceBlockingStub serviceStub;
	@Mock
	private StatisticMapper mapper;
	@Mock
	private AlfaProperties alfaProperties;

	private static final List<VorgangStatus> STATUSES_TO_COUNT = List.of(VorgangStatus.NEU, VorgangStatus.ANGENOMMEN,
			VorgangStatus.IN_BEARBEITUNG, VorgangStatus.BESCHIEDEN,
			VorgangStatus.ABGESCHLOSSEN, VorgangStatus.VERWORFEN, VorgangStatus.ZU_LOESCHEN);

	@DisplayName("Build count by vorgang status query")
	@Nested
	class TestBuildCountByVorgangStatusQuery {

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainResultName(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getResultName()).isEqualTo(String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, vorgangStatus.name()));
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainPath(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getPath()).isEqualTo(STATUS_PATH);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainGroupMethod(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getGroupMethod()).isEqualTo(GroupMethod.COUNT);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainOperator(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getOperator()).isEqualTo(GrpcQueryOperator.EQUAL);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainOperandStringValue(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getOperandStringValue()).isEqualTo(vorgangStatus.name());
		}
	}

	@Nested
	class TestBuildCountByStatusQueries {

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldCallBuildCountByVorgangStatusQuery(VorgangStatus vorgangStatus) {
			service.buildCountByStatusQueries(STATUSES_TO_COUNT);

			verify(service).buildCountByVorgangStatusQuery(vorgangStatus);
		}
	}

	@Nested
	class TestBuildCountWiedervorlageNextFristQuery {

		private final String applicationName = LoremIpsum.getInstance().getFirstName();

		@BeforeEach
		void init() {
			when(alfaProperties.getApplicationName()).thenReturn(applicationName);
		}

		@Test
		void shouldBuildQuery() {
			var query = service.buildCountWiedervorlageNextFristQuery();

			assertThat(query)
					.extracting(
							GrpcVorgangStatisticQuery::getResultName,
							GrpcVorgangStatisticQuery::getPath,
							GrpcVorgangStatisticQuery::getOperator,
							GrpcVorgangStatisticQuery::getGroupMethod)
					.contains(
							COUNT_WIEDERVORLAGE_NEXT_FRIST_RESULT_NAME,
							String.format(StatisticRemoteService.WIEDERVORLAGE_NEXT_FRIST_PATH_TEMPLATE, applicationName),
							GrpcQueryOperator.UNEQUAL,
							GroupMethod.COUNT
					);
		}
	}

	@Nested
	class TestBuildExistsWiedervorlageOverdueQuery {

		private final String applicationName = LoremIpsum.getInstance().getFirstName();

		@BeforeEach
		void init() {
			when(alfaProperties.getApplicationName()).thenReturn(applicationName);
		}

		@Test
		void shouldContainResultName() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getResultName()).isEqualTo(StatisticRemoteService.EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME);
		}

		@Test
		void shouldContainPath() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getPath()).isEqualTo(String.format(StatisticRemoteService.WIEDERVORLAGE_NEXT_FRIST_PATH_TEMPLATE, applicationName));
		}

		@Test
		void shouldContainGroupMethod() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getGroupMethod()).isEqualTo(GroupMethod.EXISTS);
		}

		@Test
		void shouldContainOperator() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getOperator()).isEqualTo(GrpcQueryOperator.LESS_THEN_OR_EQUAL_TO);
		}

		@Test
		void shouldContainOperandStringValue() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getOperandStringValue()).isEqualTo(StatisticRemoteService.OPERAND_TODAY_DATE);
		}

		@Test
		void shouldCallPropertiesToGetClientName() {
			service.buildExistsWiedervorlageOverdueQuery();

			verify(alfaProperties).getApplicationName();
		}
	}

	@DisplayName("Get vorgaenge statistic")
	@Nested
	class TestGetVorgaengeStatistics {

		private final GrpcVorgangStatisticResponse response = GrpcVorgangStatisticResponseTestFactory.create();
		private final GrpcVorgangStatisticQuery countVorgaengeWithUngeleseneNachrichtenQuery = GrpcVorgangStatisticQueryTestFactory.createBuilder()
				.setResultName("ungelesene nachrichten").build();
		private final GrpcVorgangStatisticQuery existsWiedervorlageOverdueQuery = GrpcVorgangStatisticQueryTestFactory.createBuilder()
				.setResultName("exists overdue").build();

		@Captor
		private ArgumentCaptor<GrpcVorgangStatisticRequest> grpcRequestCaptor;

		@BeforeEach
		void beforeEach() {
			when(serviceStub.getVorgangStatistic(any())).thenReturn(response);

			doReturn(existsWiedervorlageOverdueQuery).when(service).buildExistsWiedervorlageOverdueQuery();
			doReturn(countVorgaengeWithUngeleseneNachrichtenQuery).when(service).buildCountVorgaengeWithUngeleseneNachrichtenQuery();
		}

		@Test
		void shouldCallServiceStub() {
			service.getVorgaengeStatistics(Collections.emptyList());

			verify(serviceStub).getVorgangStatistic(any());
		}

		@Test
		void shouldCallBuildCountByStatusQueries() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildCountByStatusQueries(STATUSES_TO_COUNT);
		}

		@Test
		void shouldCallBuildCountByVorgangStatusQuery() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildCountWiedervorlageNextFristQuery();
		}

		@Test
		void shouldCallBuildExistsWiedervorlageOverdueQuery() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildExistsWiedervorlageOverdueQuery();
		}

		@Test
		void shouldCallBuildStatisticResult() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildStatisticResult(response);
		}

		@Test
		void shouldCallBuildCountVorgaengeWithUngeleseneNachrichtenQuery() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildCountVorgaengeWithUngeleseneNachrichtenQuery();
		}

		@Test
		void shouldAddCountVorgaengeWithUngeleseneNachrichtenQuery() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(serviceStub).getVorgangStatistic(grpcRequestCaptor.capture());
			assertThat(grpcRequestCaptor.getValue().getQueryList()).contains(countVorgaengeWithUngeleseneNachrichtenQuery);
		}
	}

	@Nested
	class TestBuildStatisticResult {

		private final String RESULT_NAME = "countByStatus";
		private final int RESULT_INT = 42;
		private GrpcVorgangStatisticResponse grpcResponse;
		private GrpcVorgangStatisticResult grpcResult;

		@BeforeEach
		void beforeEach() {
			grpcResult = GrpcVorgangStatisticResult.newBuilder()
					.setName(RESULT_NAME)
					.setResultIntValue(RESULT_INT)
					.build();
			grpcResponse = GrpcVorgangStatisticResponse.newBuilder()
					.addResult(grpcResult)
					.build();
			when(mapper.toResult(grpcResult)).thenReturn(StatisticResult.builder().name(RESULT_NAME).intValue(RESULT_INT).build());
		}

		@Test
		void shouldCallMapper() {
			service.buildStatisticResult(grpcResponse);

			verify(mapper).toResult(grpcResult);
		}

		@Test
		void shouldBuildResult() {
			var result = service.buildStatisticResult(grpcResponse);

			assertThat(result)
					.hasSize(1)
					.hasEntrySatisfying(RESULT_NAME, statisticResult -> assertThat(statisticResult.getIntValue()).isEqualTo(RESULT_INT));
		}
	}

	@Nested
	class TestBuildCountVorgaengeWithUngeleseneNachrichtenQuery {

		@Test
		void shouldHaveResultName() {
			var query = callService();

			assertThat(query.getResultName()).isEqualTo(COUNT_VORGAENGE_WITH_UNGELESENE_NACHRICHTEN_RESULT_NAME);
		}

		@Test
		void shouldHavePath() {
			var query = callService();

			assertThat(query.getPath()).isEqualTo(VORGAENGE_WITH_UNGELESENE_NACHRICHTEN_PATH);
		}

		@Test
		void shouldHaveGroupingMethod() {
			var query = callService();

			assertThat(query.getGroupMethod()).isEqualTo(GroupMethod.COUNT);
		}

		@Test
		void shouldHaveOperator() {
			var query = callService();

			assertThat(query.getOperator()).isEqualTo(GrpcQueryOperator.EQUAL);
		}

		@Test
		void shouldHaveOperandValue() {
			var query = callService();

			assertThat(query.getOperandBoolValue()).isTrue();
		}

		private GrpcVorgangStatisticQuery callService() {
			return service.buildCountVorgaengeWithUngeleseneNachrichtenQuery();
		}
	}
}